%function [image,R,G,B] = ppmread2(filename)
function image=ppmread2(filename)
% ERROR in number of outputs was a programmer's error, not a Matlab generated eror.

% image=ppmread2('DSC_0059.ppm');

%PPMREAD  Read a PPM (portable graymap) file from disk.
%
%   [X,MAP] = PPMREAD('FILENAME') reads the file 'FILENAME' and returns
%   the indexed image X and associated colormap MAP.
%
%   [R,G,B] = PPMREAD('FILENAME') reads the file 'FILENAME' and returns
%   the RGB image in the matrices R, G, and B.
%
%   If no file extension is given with the filename, the extension
%   '.ppm' is used.
%
%   See also: PPMWRITE, PBMREAD, PBMWRITE, PGMREAD, PGMWRITE.

%   1998-01-25 16:00:25  Peter J. Acklam <jacklam@math.uio.no>

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check number of input arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

error( nargchk( 1, 1, nargin ) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify output arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nargsout = nargout;
if ( nargsout == 3 | nargsout==1)
   isindexed = 0;
elseif ( nargsout == 2 )
   isindexed = 1;
else
   error( 'Wrong number of output arguments.' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check filename.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ( ~isstr( filename ) | isempty( filename ) )
   error( 'File name must be a non-empty string' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add extension to file name if necessary.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty( findstr( filename, '.' ) )
   filename = [ filename '.ppm' ];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Open input file for reading.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen( filename, 'r' );
if ( fid == -1 )
   error( [ 'Can''t open file ''' filename ''' for reading.' ] );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify file type.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ magic, count ] = fscanf( fid, '%c', 2 );      % Read first two bytes.

if ( count < 2 )
   fclose( fid );
   error( 'End of file reached while reading magic number.' );
end

if ( magic == 'P1' ) | ( magic == 'P4' )        % Bitmap.

   fclose( fid );
   if isindexed
      [ R, G ] = pbmread( filename );           % Indexed bitmap.
   else
      R = pbmread( filename );                  % Intensity bitmap.
      G = R;
      B = R;
   end
   return;

elseif ( magic == 'P2' ) | ( magic == 'P5' )    % Graymap.

   fclose( fid );
   if isindexed
      [ R , G ] = pgmread( filename );          % Indexed graymap.
   else
      R = pgmread( filename );                  % Intensity graymap.
      G = R;
      B = R;
   end
   return;

elseif ( magic == 'P3' )                % Ascii encoded pixelmap.
   isascii = 1;

elseif ( magic == 'P6' )                % Binary encoded pixelmap.
   isascii = 0;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read image size and maximum pixel value.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cols   = pnmgeti( fid );        % Number of columns.
rows   = pnmgeti( fid );        % Number of rows.
maxval = pnmgeti( fid );        % Maximum pixel value.
pixels = rows*cols;             % Total number of pixels in image.
values = 3*pixels;              % Total number of colour values.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read image data.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isascii
   [ data, count ] = fscanf( fid, '%i', values );
else
   [ data, count ] = fread( fid, values, 'uint8' );
end

fclose( fid );                  % Close file.

if ( count < values )
   disp( 'Warning: End of file reached too early.' );
end

rvals = floor( (count+2)/3 );           % Number of red values.
gvals = floor( (count+1)/3 );           % Number of green values.
bvals = floor(  count/3 );              % Number of blue values.

if isindexed                            % Indexed pixelmap.

   R = zeros( cols, rows );             % Initialize index matrix.
   R(1:pixels) = 1:pixels;              % Fill data into matrix.
   R = R';

   G = zeros( pixels, 3 );              % Initialize colour map.
   G(1:rvals,1) = data(1:3:3*rvals);    % Insert red component.
   G(1:gvals,2) = data(2:3:3*gvals);    % Insert green component.
   G(1:bvals,3) = data(3:3:3*bvals);    % Insert blue component.
   G = G/maxval;                        % Map values to [0,1] interval.

else                                    % Intensity pixelmap.

   R = zeros(cols,rows);                % Initialize red component.
   G = zeros(cols,rows);                % Initialize green component.
   B = zeros(cols,rows);                % Initialize blue component.

   R(1:rvals) = data(1:3:3*rvals);      % Insert red component.
   G(1:gvals) = data(2:3:3*gvals);      % Insert green component.
   B(1:bvals) = data(3:3:3*bvals);      % Insert blue component.

 
   R = R'/maxval;                       % Map values to [0,1] interval.
   G = G'/maxval;                       % Map values to [0,1] interval.
   B = B'/maxval;                       % Map values to [0,1] interval.
   
% image(:,:,1)=R; image(:,:,2)=G; image(:,:,3)=B; 
 image=reshape([R,G,B],size(R,1),size(R,2),3);
end
